/*
 * File:   DALI_Hack_Main.c
 * Author: Leo M Fernekes, Fernekes Designs
 *
 * Created on January 14, 2021, 5:47 PM
 */
#include "DALI_DEFINES.h"
#include "DALI_Hack_Main.h"
#include <xc.h>

void main(void) {

OSCCON = 0b11110000 ;    // PLL =ON, IRCF = 8 mhZ, Internal Oscillator
//ACTUAL CLOCK RUNS 32 Mhz - PLL IS ON

ANSELA = 0b00000000; //no analog function on PORTA
TRISA = 0b11101111; //RA4 is Comparator 2 Out TO MOSFET GATE
LATC = 0x00; //clear LATC
TRISC = 0b11110011; //RC2 = PWM1 output, RC3 = TEST OUTPUT
ANSELC = 0b00000001; // Set for DIGITAL Input, RC0 = analog comparator IN

OPTION_REG = 0b11010001; //TIMER0 clock source Fosc/2, prescale ON, pre  = 4.
//timer 0 counts at 2 Mhz. 500 ns per count



//************************************************************
//Interrupt on change enable for DALI INPUT

INTCONbits.IOCIE = 1;// enable IOC module
IOCCNbits.IOCCN5 =1; // RC5 Enable IOC for RC5 FALLING EDGES
edge_pos = 0;//searching for negative edge
IOCCFbits.IOCCF5 =0;//clear the flag

//************************************************************
 
// Setup comparator for sync input

    RA4PPSbits.RA4PPS = 0b0010;    // COMPARATOR 2 > RA4
    CM2CON0 = 0b10000111 ; //Comparitor ON, NA-OUTPUT BIT, NA, Normal Polarity, NA, HIGH SPEED, Hysteresis ENABLED, Synchronize ON
    CM2CON1 = 0b11000110; // Pos interrupts ON, Neg Interrupt ON, sel C2IN+,  sel C2IN+, NA, sel FVR 
    FVRCON = 0b10000100; //FRV ON, Vfvr =1 V to comparator
    TMR1H =0; //clear timer 1
    TMR1L =0; //
    T1CON = 0b10000101; // clock source ext T1CKI,  PRE= 1, NA,  No sync, NA,  TIMER = ON

 //************************************************************

 // prevent generating any PWM output before validating that the LED current control engine is running.
 // If no LEDs are connected to driver, generating PWM could destroy the driver circuit!!!   
    
    while (! (TMR1L & 0x80))
    {
    // wait for 128 cycles of clock before activating the PWM
    NOP();    
    }

    
 //************************************************************
 //at 32 Mhz, PWM period = 1 ms., 1 kHz.   
    //PWM GERNERATOR SETUP- #1 MASTER
    load_PWM_1(0x8FFF); //prime the system LIGHTS ON FULL - default
    PWM1CON = 0b10010000; //turn on PWM normal mode, polarity- active LOW
    PWM1OFCON = 0b00000001 ; //Start in free-run mode 
    PWM1CLKCON =0;       
    PWM1LDCON = 0b00000000; //SET LOAD ON TRIGGER OFF
    PWM1DCL = 0;
    PWM1DCH = 0; 
    PWM1PHL = 0;
    PWM1PHH = 0;
    PWM1PRL = 0xFF;
    PWM1PRH = 0x8F;
    PWM1OFL = 0;
    PWM1OFH = 0;
    RC2PPSbits.RC2PPS = 0b0011;    // PWM1>RC2

 //*******************************************************************************     
    
    
    
INTCONbits.PEIE =1; //Peripheral ints ENABLED
INTCONbits.GIE =1; // GLOBAL Ints ON

 //******************************************************************************* 
    
//MAIN LOOP START:
while (1) //loop forever...
{


    
    if (new_data) //test for new data from DALI INPUT   
    {
        
    new_data =0;// clear the flag
     
    //process the new address and DALI data 
        if (address == 0xFE) //DALI Broadcast Address!
        {   
        unsigned int i;    
         
        i = (data * LED_SCALE); //scale input to PWM register
        
        if(data)//add min brightness level for NON zero values
        { 
            i += LED_OFFSET ;// add offset- min level
        }
        PWM1DCH = ((i >> 8) & 0xff);   //data to PWM register
        PWM1DCL = (i & 0xff);   //data to PWM register
                
        PWM1LDCONbits.LDA = 1; // setup to load buffers
         
         
            if (PWM1DCH & 0xC0) // test for duty more than 50 %
            {
            CM2CON0bits.C2SYNC = 1; //enable clock synchronization
            }        
            else
            {
            CM2CON0bits.C2SYNC = 0; // NO sync
            }
        }    
    }               
      



} //WHILE 1   
    
    

    
}//MAIN

 //******************************************************************************* 

void __interrupt() IOC_Interrupt(void)

{
unsigned char i;

 if (INTCONbits.IOCIE && IOCCFbits.IOCCF5)// test if IOC mode is active 
 //   if (0)//TEST
 {
  IOCCFbits.IOCCF5 =0;//clear the interrupt flag  

        if (idle_flag)//first time - begin reception
        {
        shift_register =0;
        byte_count= 17;
        idle_flag = 0;
        }
  
//setup for interrupt in 3/4 bit time      
  
 timer_0_counts = _3_4_BIT_TIME ;// load the delay Register
 TMR0 = 0xFF - (timer_0_counts & 0xFF); // load the timer with the lower 8 bits 
 timer_0_counts =  (timer_0_counts & 0xFF00);// mask-off lower bits   
 INTCONbits.T0IF = 0; //clear flag
 INTCONbits.T0IE = 1; //enable timer 0 interrupts
 INTCONbits.IOCIE =0; //turn off IOC mode
 inter_bit_time_out = 0; //not inter-bit timer mode

 
 return;     //exit the ISR!!!
  
 }   
 
 
 //TIMER 0 interrupts here!*******************************************************


  INTCONbits.TMR0IF = 0; //clear flag  

  if ( timer_0_counts )
  {
  timer_0_counts -= 0x100; //subtract a cycle   
  return;   //not done with timer delay 
  }
  //check for time out lapsed
  if(inter_bit_time_out)
  { //if lapse- reset all to idle state
  idle_flag = 1; //back to idle mode
  inter_bit_time_out =0; // reset
  edge_pos = 0 ; //falling EDGE
  IOCCNbits.IOCCN5 =1;   //setup to look for FALLING edge 
  IOCCPbits.IOCCP5 =0;  
  IOCCFbits.IOCCF5 =0;//clear the interrupt flag
  INTCONbits.IOCIE =1; //enable IOC interrupts  
  INTCONbits.T0IE = 0; //shutdown timer 0 interrupts 
  return;
  }
  
  
 //timer period has lapsed - continue 

  i = ( ! PORTCbits.RC5); //read the state of the port- LOW = logic ONE
 
    byte_count--;
    
    if (byte_count < 16 )
    {
    shift_register <<= 1;
    shift_register |= (!edge_pos) ; //shift in the current edge state

    }

          
         if (! byte_count) //byte_count = 0, end of byte
         {
         data = shift_register & 0x00FF ; // mask off data payload    
         address = (shift_register & 0xFF00) >> 8; // shift address to low byte- save in address     
         idle_flag = 1; //back to idle mode
         inter_bit_time_out =0; // reset
         edge_pos = 0 ; //falling EDGE
         INTCONbits.T0IE = 0; //shutdown timer 0 interrupts 
         IOCCNbits.IOCCN5 =1;   //setup to look for FALLING edge 
         IOCCPbits.IOCCP5 =0;  
         IOCCFbits.IOCCF5 =0;//clear the interrupt flag
         INTCONbits.IOCIE =1; //enable IOC interrupts
         
         
         new_data= 1;// set the flag

         return;   
         }    
              
        if (! i) //check polarity of data bit
        {
        //its high- setup to look for falling edge
        IOCCNbits.IOCCN5 =1;
        IOCCPbits.IOCCP5 =0;
        edge_pos = 0 ; //FALLING EDGE
        }   
        else
            
        {
        //its low- setup to look for rising edge        
        IOCCNbits.IOCCN5 =0;
        IOCCPbits.IOCCP5 =1; 
        edge_pos = 1 ; //RISING EDGE
        }   
    //setup inter-bit timeout 
    inter_bit_time_out = 1;
    timer_0_counts = _1_5_BIT_TIME ;// load the delay Register 1.5 bit times.
    TMR0 = 0xFF - (timer_0_counts & 0xFF); // load the timer with the lower 8 bits 
    timer_0_counts =  (timer_0_counts & 0xFF00);// mask-off lower bits   
    
  IOCCFbits.IOCCF5 =0;//clear the interrupt flag
  INTCONbits.IOCIE =1; //enable IOC interrupts
  
  

    
    
}//ISR



//**************************************************************************
void load_PWM_1 (unsigned int val) 

{
    

PWM1DCL = val & 0x00FF;
PWM1DCH = (val & 0xFF00) >> 8 ;   
PWM1LDCONbits.LDA = 1; // setup to load buffers


}
